@@ -21,18 +21,18 @@
 
 /**
  * SECTION:element-shout2send
+ * @title: shout2send
  *
  * shout2send pushes a media stream to an Icecast server
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 uridecodebin uri=file:///path/to/audiofile ! audioconvert ! vorbisenc ! oggmux ! shout2send mount=/stream.ogg port=8000 username=source password=somepassword ip=server_IP_address_or_hostname
  * ]| This pipeline demuxes, decodes, re-encodes and re-muxes an audio
  * media file into oggvorbis and sends the resulting stream to an Icecast
  * server. Properties mount, port, username and password are all server-config
  * dependent.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,12 +43,17 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "gst/gst-i18n-plugin.h"
+#include <glib/gi18n-lib.h>
+
+#include <gst/glib-compat-private.h>
+
+#ifndef HAVE_SHOUT_2_4_6_OR_NEWER
+#define shout_set_metadata_utf8 shout_set_metadata
+#endif
 
 GST_DEBUG_CATEGORY_STATIC (shout2_debug);
 #define GST_CAT_DEFAULT shout2_debug
 
-
 enum
 {
   SIGNAL_CONNECTION_PROBLEM,    /* FIXME 2.0: remove this */
@@ -72,33 +77,35 @@ enum
   ARG_MOUNT,                    /* mountpoint of stream (icecast only) */
   ARG_URL,                      /* the stream's homepage URL */
 
-  ARG_TIMEOUT                   /* The max amount of time to wait for
+  ARG_TIMEOUT,                  /* The max amount of time to wait for
                                    network activity */
+  ARG_SEND_TITLE_INFO,          /* If stream song title updates should be made */
+  ARG_USERAGENT                 /* User-Agent setting */
 };
 
 #define DEFAULT_IP           "127.0.0.1"
 #define DEFAULT_PORT         8000
 #define DEFAULT_PASSWORD     "hackme"
 #define DEFAULT_USERNAME     "source"
-#define DEFAULT_PUBLIC     FALSE
+#define DEFAULT_PUBLIC       FALSE
 #define DEFAULT_STREAMNAME   ""
 #define DEFAULT_DESCRIPTION  ""
+#define DEFAULT_USERAGENT    "GStreamer {VERSION}"
 #define DEFAULT_GENRE        ""
 #define DEFAULT_MOUNT        ""
 #define DEFAULT_URL          ""
 #define DEFAULT_PROTOCOL     SHOUT2SEND_PROTOCOL_HTTP
 #define DEFAULT_TIMEOUT      10000
+#define DEFAULT_SEND_TITLE_INFO TRUE
+
+#define SHOUT2SEND_CAPS "application/ogg; audio/ogg; video/ogg; "\
+    "audio/mpeg, mpegversion = (int) 1, layer = (int) [ 1, 3 ]; " \
+    "video/webm; audio/webm"
 
-#ifdef SHOUT_FORMAT_WEBM
-#define WEBM_CAPS "; video/webm; audio/webm"
-#else
-#define WEBM_CAPS ""
-#endif
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("application/ogg; audio/ogg; video/ogg; "
-        "audio/mpeg, mpegversion = (int) 1, layer = (int) [ 1, 3 ]" WEBM_CAPS));
+    GST_STATIC_CAPS (SHOUT2SEND_CAPS));
 
 static void gst_shout2send_finalize (GstShout2send * shout2send);
 
@@ -144,6 +151,15 @@ gst_shout2send_protocol_get_type (void)
 #define gst_shout2send_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstShout2send, gst_shout2send, GST_TYPE_BASE_SINK,
     G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL));
+#ifdef ENABLE_NLS
+#define _do_init \
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);\
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#else /* ENABLE_NLS */
+#define _do_init
+#endif
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (shout2send, "shout2send", GST_RANK_NONE,
+    GST_TYPE_SHOUT2SEND, _do_init);
 
 static void
 gst_shout2send_class_init (GstShout2sendClass * klass)
@@ -151,6 +167,7 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
   GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
   GstBaseSinkClass *gstbasesink_class;
+  GstPadTemplate *tmpl;
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
@@ -201,6 +218,35 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
           GST_TYPE_SHOUT_PROTOCOL, DEFAULT_PROTOCOL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstShout2send:send-title-info
+   *
+   * Update stream metadata with song title and artist information
+   *
+   * Since: 1.22
+   **/
+
+  g_object_class_install_property (G_OBJECT_CLASS (klass),
+      ARG_SEND_TITLE_INFO,
+      g_param_spec_boolean ("send-title-info", "send-title-info",
+          "Update stream metadata with song title and artist information",
+          DEFAULT_SEND_TITLE_INFO, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstShout2send:user-agent
+   *
+   * User agent of the source
+   *
+   * If the string contains `{VERSION}` that will be replaced with the
+   * GStreamer version at runtime (since GStreamer 1.24).
+   *
+   * Since: 1.22
+   **/
+
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_USERAGENT,
+      g_param_spec_string ("user-agent", "user-agent",
+          "User agent of the source", DEFAULT_USERAGENT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /* icecast only */
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MOUNT,
@@ -220,9 +266,7 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
   /* signals */
   gst_shout2send_signals[SIGNAL_CONNECTION_PROBLEM] =
       g_signal_new ("connection-problem", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_CLEANUP, G_STRUCT_OFFSET (GstShout2sendClass,
-          connection_problem), NULL, NULL, g_cclosure_marshal_VOID__INT,
-      G_TYPE_NONE, 1, G_TYPE_INT);
+      0, 0, NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_INT);
 
   gstbasesink_class->start = GST_DEBUG_FUNCPTR (gst_shout2send_start);
   gstbasesink_class->stop = GST_DEBUG_FUNCPTR (gst_shout2send_stop);
@@ -233,7 +277,8 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
   gstbasesink_class->event = GST_DEBUG_FUNCPTR (gst_shout2send_event);
   gstbasesink_class->set_caps = GST_DEBUG_FUNCPTR (gst_shout2send_setcaps);
 
-  gst_element_class_add_static_pad_template (gstelement_class, &sink_template);
+  tmpl = gst_static_pad_template_get (&sink_template);
+  gst_element_class_add_pad_template (gstelement_class, tmpl);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "Icecast network sink",
@@ -243,6 +288,8 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
       "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
 
   GST_DEBUG_CATEGORY_INIT (shout2_debug, "shout2", 0, "shout2send element");
+
+  gst_type_mark_as_plugin_api (GST_TYPE_SHOUT_PROTOCOL, 0);
 }
 
 static void
@@ -266,12 +313,15 @@ gst_shout2send_init (GstShout2send * shout2send)
   shout2send->timeout = DEFAULT_TIMEOUT;
 
   shout2send->format = -1;
+  shout2send->usage = SHOUT_USAGE_UNKNOWN;
   shout2send->tags = gst_tag_list_new_empty ();
   shout2send->conn = NULL;
   shout2send->connected = FALSE;
   shout2send->songmetadata = NULL;
   shout2send->songartist = NULL;
   shout2send->songtitle = NULL;
+  shout2send->send_title_info = DEFAULT_SEND_TITLE_INFO;
+  shout2send->user_agent = g_strdup (DEFAULT_USERAGENT);
 }
 
 static void
@@ -285,6 +335,7 @@ gst_shout2send_finalize (GstShout2send * shout2send)
   g_free (shout2send->genre);
   g_free (shout2send->mount);
   g_free (shout2send->url);
+  g_free (shout2send->user_agent);
 
   gst_tag_list_unref (shout2send->tags);
 
@@ -334,6 +385,10 @@ set_shout_metadata (const GstTagList * list, const gchar * tag,
   if (*shout_metadata != NULL)
     g_free (*shout_metadata);
 
+  if (!shout2send->send_title_info) {
+    *shout_metadata = NULL;
+    return;
+  }
 
   if (*song_title && *song_artist) {
     *shout_metadata = g_strdup_printf ("%s - %s", *song_artist, *song_title);
@@ -406,13 +461,21 @@ gst_shout2send_event (GstBaseSink * sink, GstEvent * event)
             set_shout_metadata, shout2send);
         if (shout2send->songmetadata && shout2send->connected) {
           shout_metadata_t *pmetadata;
+          int shout_ret;
 
           GST_DEBUG_OBJECT (shout2send, "metadata now: %s",
               shout2send->songmetadata);
 
           pmetadata = shout_metadata_new ();
-          shout_metadata_add (pmetadata, "song", shout2send->songmetadata);
-          shout_set_metadata (shout2send->conn, pmetadata);
+          shout_ret =
+              shout_metadata_add (pmetadata, "song", shout2send->songmetadata);
+          if (shout_ret == SHOUTERR_SUCCESS) {
+            shout_ret = shout_set_metadata_utf8 (shout2send->conn, pmetadata);
+            if (shout_ret != SHOUTERR_SUCCESS) {
+              GST_WARNING_OBJECT (shout2send, "Failed to set metadata: %s",
+                  shout_get_error (shout2send->conn));
+            }
+          }
           shout_metadata_free (pmetadata);
         }
       }
@@ -431,13 +494,27 @@ gst_shout2send_event (GstBaseSink * sink, GstEvent * event)
   return ret;
 }
 
+static gboolean
+gst_shout2send_set_meta (GstShout2send * sink, const char *meta,
+    const char *val)
+{
+  GST_DEBUG_OBJECT (sink, "setting %s: %s", meta, val);
+
+  if (shout_set_meta (sink->conn, meta, val) == SHOUTERR_SUCCESS)
+    return TRUE;
+
+  GST_ELEMENT_ERROR (sink, LIBRARY, SETTINGS, (NULL),
+      ("Error setting %s: %s", meta, shout_get_error (sink->conn)));
+
+  return FALSE;
+}
+
 static gboolean
 gst_shout2send_start (GstBaseSink * basesink)
 {
   GstShout2send *sink = GST_SHOUT2SEND (basesink);
   const gchar *cur_prop;
   gshort proto = 3;
-  gchar *version_string;
 
   GST_DEBUG_OBJECT (sink, "starting");
 
@@ -481,20 +558,15 @@ gst_shout2send_start (GstBaseSink * basesink)
           (sink->ispublic ? 1 : 0)) != SHOUTERR_SUCCESS)
     goto set_failed;
 
-  cur_prop = "streamname";
-  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->streamname);
-  if (shout_set_name (sink->conn, sink->streamname) != SHOUTERR_SUCCESS)
-    goto set_failed;
+  if (!gst_shout2send_set_meta (sink, SHOUT_META_NAME, sink->streamname))
+    goto set_meta_failed;
 
-  cur_prop = "description";
-  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->description);
-  if (shout_set_description (sink->conn, sink->description) != SHOUTERR_SUCCESS)
-    goto set_failed;
+  if (!gst_shout2send_set_meta (sink, SHOUT_META_DESCRIPTION,
+          sink->description))
+    goto set_meta_failed;
 
-  cur_prop = "genre";
-  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->genre);
-  if (shout_set_genre (sink->conn, sink->genre) != SHOUTERR_SUCCESS)
-    goto set_failed;
+  if (!gst_shout2send_set_meta (sink, SHOUT_META_GENRE, sink->genre))
+    goto set_meta_failed;
 
   cur_prop = "mount";
   GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, sink->mount);
@@ -506,15 +578,16 @@ gst_shout2send_start (GstBaseSink * basesink)
   if (shout_set_user (sink->conn, sink->username) != SHOUTERR_SUCCESS)
     goto set_failed;
 
-  version_string = gst_version_string ();
   cur_prop = "agent";
-  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, version_string);
-  if (shout_set_agent (sink->conn, version_string) != SHOUTERR_SUCCESS) {
-    g_free (version_string);
+  GString *user_agent = g_string_new (sink->user_agent);
+  g_string_replace (user_agent, "{VERSION}", PACKAGE_VERSION, 0);
+  GST_DEBUG_OBJECT (sink, "setting %s: %s", cur_prop, user_agent->str);
+  if (shout_set_agent (sink->conn, user_agent->str) != SHOUTERR_SUCCESS) {
+    g_string_free (user_agent, TRUE);
     goto set_failed;
   }
+  g_string_free (user_agent, TRUE);
 
-  g_free (version_string);
   return TRUE;
 
 /* ERROR */
@@ -522,6 +595,12 @@ set_failed:
   {
     GST_ELEMENT_ERROR (sink, LIBRARY, SETTINGS, (NULL),
         ("Error setting %s: %s", cur_prop, shout_get_error (sink->conn)));
+    /* fallthrough */
+  }
+set_meta_failed:
+  {
+    shout_free (sink->conn);
+    sink->conn = NULL;
     return FALSE;
   }
 }
@@ -541,8 +620,17 @@ gst_shout2send_connect (GstShout2send * sink)
   if (shout_set_nonblocking (sink->conn, 1) != SHOUTERR_SUCCESS)
     goto could_not_set_nonblocking;
 
-  if (shout_set_format (sink->conn, sink->format) != SHOUTERR_SUCCESS)
-    goto could_not_set_format;
+  {
+#ifdef HAVE_SHOUT_2_4_6_OR_NEWER
+    ret =
+        shout_set_content_format (sink->conn, sink->format, sink->usage, NULL);
+#else
+    ret = shout_set_format (sink->conn, sink->format);
+#endif
+
+    if (ret != SHOUTERR_SUCCESS)
+      goto could_not_set_format;
+  }
 
   GST_DEBUG_OBJECT (sink, "connecting");
 
@@ -550,7 +638,9 @@ gst_shout2send_connect (GstShout2send * sink)
   ret = shout_open (sink->conn);
 
   /* wait for connection or timeout */
-  while (ret == SHOUTERR_BUSY) {
+  /* starting with libshout 2.4.2, shout_open() has broken API + ABI and
+   * can also return SHOUTERR_RETRY (a new define) to mean "try again" */
+  while (ret == SHOUTERR_BUSY || ret == SHOUTERR_RETRY) {
     if (gst_util_get_timestamp () - start_ts > sink->timeout * GST_MSECOND) {
       goto connection_timeout;
     }
@@ -582,8 +672,14 @@ gst_shout2send_connect (GstShout2send * sink)
 
     GST_DEBUG_OBJECT (sink, "shout metadata now: %s", sink->songmetadata);
     pmetadata = shout_metadata_new ();
-    shout_metadata_add (pmetadata, "song", sink->songmetadata);
-    shout_set_metadata (sink->conn, pmetadata);
+    ret = shout_metadata_add (pmetadata, "song", sink->songmetadata);
+    if (ret == SHOUTERR_SUCCESS) {
+      ret = shout_set_metadata_utf8 (sink->conn, pmetadata);
+      if (ret != SHOUTERR_SUCCESS) {
+        GST_WARNING_OBJECT (sink, "Failed to set metadata: %s",
+            shout_get_error (sink->conn));
+      }
+    }
     shout_metadata_free (pmetadata);
   }
 
@@ -652,6 +748,7 @@ gst_shout2send_stop (GstBaseSink * basesink)
 
   sink->connected = FALSE;
   sink->format = -1;
+  sink->usage = SHOUT_USAGE_UNKNOWN;
 
   return TRUE;
 }
@@ -802,48 +899,55 @@ gst_shout2send_set_property (GObject * object, guint prop_id,
 
     case ARG_IP:
       g_free (shout2send->ip);
-      shout2send->ip = g_strdup (g_value_get_string (value));
+      shout2send->ip = g_value_dup_string (value);
       break;
     case ARG_PORT:
       shout2send->port = g_value_get_int (value);
       break;
     case ARG_PASSWORD:
       g_free (shout2send->password);
-      shout2send->password = g_strdup (g_value_get_string (value));
+      shout2send->password = g_value_dup_string (value);
       break;
     case ARG_USERNAME:
       g_free (shout2send->username);
-      shout2send->username = g_strdup (g_value_get_string (value));
+      shout2send->username = g_value_dup_string (value);
       break;
     case ARG_PUBLIC:
       shout2send->ispublic = g_value_get_boolean (value);
       break;
     case ARG_STREAMNAME:       /* Name of the stream */
       g_free (shout2send->streamname);
-      shout2send->streamname = g_strdup (g_value_get_string (value));
+      shout2send->streamname = g_value_dup_string (value);
       break;
     case ARG_DESCRIPTION:      /* Description of the stream */
       g_free (shout2send->description);
-      shout2send->description = g_strdup (g_value_get_string (value));
+      shout2send->description = g_value_dup_string (value);
       break;
     case ARG_GENRE:            /* Genre of the stream */
       g_free (shout2send->genre);
-      shout2send->genre = g_strdup (g_value_get_string (value));
+      shout2send->genre = g_value_dup_string (value);
       break;
     case ARG_PROTOCOL:         /* protocol to connect with */
       shout2send->protocol = g_value_get_enum (value);
       break;
     case ARG_MOUNT:            /* mountpoint of stream (icecast only) */
       g_free (shout2send->mount);
-      shout2send->mount = g_strdup (g_value_get_string (value));
+      shout2send->mount = g_value_dup_string (value);
       break;
     case ARG_URL:              /* the stream's homepage URL */
       g_free (shout2send->url);
-      shout2send->url = g_strdup (g_value_get_string (value));
+      shout2send->url = g_value_dup_string (value);
       break;
     case ARG_TIMEOUT:
       shout2send->timeout = g_value_get_uint (value);
       break;
+    case ARG_SEND_TITLE_INFO:
+      shout2send->send_title_info = g_value_get_boolean (value);
+      break;
+    case ARG_USERAGENT:
+      g_free (shout2send->user_agent);
+      shout2send->user_agent = g_value_dup_string (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -895,6 +999,12 @@ gst_shout2send_get_property (GObject * object, guint prop_id,
     case ARG_TIMEOUT:
       g_value_set_uint (value, shout2send->timeout);
       break;
+    case ARG_SEND_TITLE_INFO:
+      g_value_set_boolean (value, shout2send->send_title_info);
+      break;
+    case ARG_USERAGENT:
+      g_value_set_string (value, shout2send->user_agent);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -916,12 +1026,23 @@ gst_shout2send_setcaps (GstBaseSink * basesink, GstCaps * caps)
 
   if (!strcmp (mimetype, "audio/mpeg")) {
     shout2send->format = SHOUT_FORMAT_MP3;
+    shout2send->usage = SHOUT_USAGE_AUDIO;
   } else if (g_str_has_suffix (mimetype, "/ogg")) {
     shout2send->format = SHOUT_FORMAT_OGG;
-#ifdef SHOUT_FORMAT_WEBM
+    if (g_str_has_prefix (mimetype, "audio/"))
+      shout2send->usage = SHOUT_USAGE_AUDIO;
+    else if (g_str_has_prefix (mimetype, "video/"))
+      shout2send->usage = SHOUT_USAGE_VISUAL | SHOUT_USAGE_AUDIO;
+    else
+      shout2send->usage = SHOUT_USAGE_UNKNOWN;
   } else if (g_str_has_suffix (mimetype, "/webm")) {
     shout2send->format = SHOUT_FORMAT_WEBM;
-#endif
+    if (g_str_has_prefix (mimetype, "audio/"))
+      shout2send->usage = SHOUT_USAGE_AUDIO;
+    else if (g_str_has_prefix (mimetype, "video/"))
+      shout2send->usage = SHOUT_USAGE_VISUAL | SHOUT_USAGE_AUDIO;
+    else
+      shout2send->usage = SHOUT_USAGE_UNKNOWN;
   } else {
     ret = FALSE;
   }
@@ -932,13 +1053,7 @@ gst_shout2send_setcaps (GstBaseSink * basesink, GstCaps * caps)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-#ifdef ENABLE_NLS
-  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-#endif /* ENABLE_NLS */
-
-  return gst_element_register (plugin, "shout2send", GST_RANK_NONE,
-      GST_TYPE_SHOUT2SEND);
+  return GST_ELEMENT_REGISTER (shout2send, plugin);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
